unit plusdbs;
{ TDBPlusSpinEdit component, derived from TPlusSpinEdit }
{ by Electro-Concept mauricie }

interface
uses PlusSpEd, Messages, Classes, Controls, DB, DBTables, dbCtrls, StdCtrls;

type TDBPlusSpinEdit = class(TPlusSpinEdit)
                         private
                         FDataLink: TFieldDataLink;
                         fInChange, fInDataChange: Boolean;
                         procedure DataChange(Sender: TObject);
                         procedure UpdateData(Sender: TObject);
                         function  GetDataField: string;
                         procedure SetDataField(const fld: string);
                         function  GetDataSource: TDataSource;
                         procedure SetDataSource(Value: TDataSource);
                         function  GetField: TField;
                         function  GetReadOnly: Boolean;
                         procedure SetReadOnly(Value: Boolean);

                         protected
                         procedure Notification(AComponent: TComponent; Operation: TOperation); override;
                         procedure Change; override;

                         procedure CMEnter(var Message: TCMEnter); message CM_ENTER;
                         procedure CMExit(var Message: TCMExit); message CM_EXIT;

                         public
                         constructor Create(AOwner: TComponent); override;
                         destructor Destroy; override;
                         property Field: TField read GetField;

                         published
                         property DataSource: TDataSource read GetDataSource write SetDataSource;
                         property DataField: string read GetDataField write SetDataField;
                         property ReadOnly: Boolean read GetReadOnly write SetReadOnly;
                         end;


procedure Register;

implementation


constructor TDBPlusSpinEdit.Create(AOwner: TComponent);
  begin
  inherited Create(AOwner);
  FDataLink := TFieldDataLink.Create;
  {FDataLink.Control := Self;}
  FDataLink.OnDataChange := DataChange;
  {FDataLink.OnEditingChange := EditingChange;}
  FDataLink.OnUpdateData := UpdateData;
  end;

destructor TDBPlusSpinEdit.Destroy;
  begin
  FDataLink.Free;
  FDataLink:= nil;
  inherited Destroy
  end;

procedure TDBPlusSpinEdit.Notification(AComponent: TComponent;
  Operation: TOperation);
  begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
  end;

function TDBPlusSpinEdit.GetDataSource: TDataSource;
  begin
  Result := FDataLink.DataSource;
  end;

procedure TDBPlusSpinEdit.SetDataSource(Value: TDataSource);
  begin
  FDataLink.DataSource := Value;
{  if Value <> nil then Value.FreeNotification(Self); }
  end;

function TDBPlusSpinEdit.GetDataField: string;
  begin
  Result := FDataLink.FieldName;
  end;

procedure TDBPlusSpinEdit.SetDataField(const fld: string);
  begin
  FDataLink.FieldName := fld;
  end;


function TDBPlusSpinEdit.GetField: TField;
  begin
  Result := FDataLink.Field;
  end;

function TDBPlusSpinEdit.GetReadOnly: Boolean;
  begin
  Result := FDataLink.ReadOnly;
  end;

procedure TDBPlusSpinEdit.SetReadOnly(Value: Boolean);
  begin
  FDataLink.ReadOnly := Value;
  end;

procedure TDBPlusSpinEdit.CMEnter(var Message: TCMEnter);
begin
  FDataLink.Reset;
  inherited;
end;


procedure TDBPlusSpinEdit.CMExit(var Message: TCMExit);
  begin
  FDataLink.UpdateRecord;
  FDataLink.Reset;
  inherited
  end;

procedure TDBPlusSpinEdit.DataChange(Sender: TObject);
  begin
  if fInChange then Exit;
  fInDataChange:= True;
  if FDataLink.Field <> nil then
    Value:= FDataLink.Field.AsFloat;
  fInDataChange:= False
  end;

procedure TDBPlusSpinEdit.Change;
  begin
  if (not fInDataChange) and (FDataLink<>nil) then
    begin
    FDataLink.Edit;
    FDataLink.Modified
    end
  else fInChange:= True;
  inherited Change;
  fInChange:= False
  end;

procedure TDBPlusSpinEdit.UpdateData(Sender: TObject);
  begin
  if FDataLink.Field<>nil then
    FDataLink.Field.AsFloat:= Value
  end;

procedure Register;
  begin
  RegisterComponents('ECM', [TDBPlusSpinEdit])
  end;


end.
